# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/orchestrator.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cogment/api/orchestrator.proto',
  package='cogment',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1e\x63ogment/api/orchestrator.proto\x12\x07\x63ogment\x1a\x18\x63ogment/api/common.proto\"\x17\n\x15GetTrialParamsRequest\";\n\x13GetTrialParamsReply\x12$\n\x06params\x18\x01 \x01(\x0b\x32\x14.cogment.TrialParams\")\n\x15TrialHeartbeatRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\"\x15\n\x13TrialHeartbeatReply\"J\n\x11TrialStartRequest\x12$\n\x06\x63onfig\x18\x01 \x01(\x0b\x32\x14.cogment.TrialConfig\x12\x0f\n\x07user_id\x18\x02 \x01(\t\"6\n\x10TrialJoinRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12\x10\n\x08\x61\x63tor_id\x18\x02 \x01(\x05\"\xa2\x01\n\x0fTrialStartReply\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12\x10\n\x08\x61\x63tor_id\x18\x02 \x01(\x05\x12*\n\x0chuman_config\x18\x03 \x01(\x0b\x32\x14.cogment.ActorConfig\x12)\n\x0bobservation\x18\x04 \x01(\x0b\x32\x14.cogment.Observation\x12\x14\n\x0c\x61\x63tor_counts\x18\x05 \x03(\x05\"#\n\x0fTrialEndRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\"\x0f\n\rTrialEndReply\"Y\n\x12TrialActionRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12\x10\n\x08\x61\x63tor_id\x18\x02 \x01(\x05\x12\x1f\n\x06\x61\x63tion\x18\x03 \x01(\x0b\x32\x0f.cogment.Action\"\xa3\x01\n\x10TrialActionReply\x12)\n\x0bobservation\x18\x01 \x01(\x0b\x32\x14.cogment.Observation\x12\x1f\n\x06reward\x18\x02 \x01(\x0b\x32\x0f.cogment.Reward\x12\x15\n\rtrial_is_over\x18\x03 \x01(\x08\x12,\n\x08messages\x18\x04 \x01(\x0b\x32\x1a.cogment.MessageCollection\"N\n\x14TrialFeedbackRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12$\n\tfeedbacks\x18\x02 \x03(\x0b\x32\x11.cogment.Feedback\"\x14\n\x12TrialFeedbackReply\"K\n\x13TrialMessageRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12\"\n\x08messages\x18\x02 \x03(\x0b\x32\x10.cogment.Message\"\x13\n\x11TrialMessageReply2\xc8\x05\n\x05Trial\x12P\n\x0eGetTrialParams\x12\x1e.cogment.GetTrialParamsRequest\x1a\x1c.cogment.GetTrialParamsReply\"\x00\x12?\n\x05Start\x12\x1a.cogment.TrialStartRequest\x1a\x18.cogment.TrialStartReply\"\x00\x12=\n\x04Join\x12\x19.cogment.TrialJoinRequest\x1a\x18.cogment.TrialStartReply\"\x00\x12\x39\n\x03\x45nd\x12\x18.cogment.TrialEndRequest\x1a\x16.cogment.TrialEndReply\"\x00\x12\x42\n\x06\x41\x63tion\x12\x1b.cogment.TrialActionRequest\x1a\x19.cogment.TrialActionReply\"\x00\x12L\n\x0c\x41\x63tionStream\x12\x1b.cogment.TrialActionRequest\x1a\x19.cogment.TrialActionReply\"\x00(\x01\x30\x01\x12K\n\tHeartbeat\x12\x1e.cogment.TrialHeartbeatRequest\x1a\x1c.cogment.TrialHeartbeatReply\"\x00\x12L\n\x0cGiveFeedback\x12\x1d.cogment.TrialFeedbackRequest\x1a\x1b.cogment.TrialFeedbackReply\"\x00\x12:\n\x07Version\x12\x17.cogment.VersionRequest\x1a\x14.cogment.VersionInfo\"\x00\x12I\n\x0bSendMessage\x12\x1c.cogment.TrialMessageRequest\x1a\x1a.cogment.TrialMessageReply\"\x00\x62\x06proto3')
  ,
  dependencies=[cogment_dot_api_dot_common__pb2.DESCRIPTOR,])




_GETTRIALPARAMSREQUEST = _descriptor.Descriptor(
  name='GetTrialParamsRequest',
  full_name='cogment.GetTrialParamsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=69,
  serialized_end=92,
)


_GETTRIALPARAMSREPLY = _descriptor.Descriptor(
  name='GetTrialParamsReply',
  full_name='cogment.GetTrialParamsReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='params', full_name='cogment.GetTrialParamsReply.params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=94,
  serialized_end=153,
)


_TRIALHEARTBEATREQUEST = _descriptor.Descriptor(
  name='TrialHeartbeatRequest',
  full_name='cogment.TrialHeartbeatRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.TrialHeartbeatRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=155,
  serialized_end=196,
)


_TRIALHEARTBEATREPLY = _descriptor.Descriptor(
  name='TrialHeartbeatReply',
  full_name='cogment.TrialHeartbeatReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=198,
  serialized_end=219,
)


_TRIALSTARTREQUEST = _descriptor.Descriptor(
  name='TrialStartRequest',
  full_name='cogment.TrialStartRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='config', full_name='cogment.TrialStartRequest.config', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='cogment.TrialStartRequest.user_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=221,
  serialized_end=295,
)


_TRIALJOINREQUEST = _descriptor.Descriptor(
  name='TrialJoinRequest',
  full_name='cogment.TrialJoinRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.TrialJoinRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='cogment.TrialJoinRequest.actor_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=297,
  serialized_end=351,
)


_TRIALSTARTREPLY = _descriptor.Descriptor(
  name='TrialStartReply',
  full_name='cogment.TrialStartReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.TrialStartReply.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='cogment.TrialStartReply.actor_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='human_config', full_name='cogment.TrialStartReply.human_config', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='observation', full_name='cogment.TrialStartReply.observation', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_counts', full_name='cogment.TrialStartReply.actor_counts', index=4,
      number=5, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=354,
  serialized_end=516,
)


_TRIALENDREQUEST = _descriptor.Descriptor(
  name='TrialEndRequest',
  full_name='cogment.TrialEndRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.TrialEndRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=518,
  serialized_end=553,
)


_TRIALENDREPLY = _descriptor.Descriptor(
  name='TrialEndReply',
  full_name='cogment.TrialEndReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=555,
  serialized_end=570,
)


_TRIALACTIONREQUEST = _descriptor.Descriptor(
  name='TrialActionRequest',
  full_name='cogment.TrialActionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.TrialActionRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='cogment.TrialActionRequest.actor_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='action', full_name='cogment.TrialActionRequest.action', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=572,
  serialized_end=661,
)


_TRIALACTIONREPLY = _descriptor.Descriptor(
  name='TrialActionReply',
  full_name='cogment.TrialActionReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='observation', full_name='cogment.TrialActionReply.observation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reward', full_name='cogment.TrialActionReply.reward', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='trial_is_over', full_name='cogment.TrialActionReply.trial_is_over', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='messages', full_name='cogment.TrialActionReply.messages', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=664,
  serialized_end=827,
)


_TRIALFEEDBACKREQUEST = _descriptor.Descriptor(
  name='TrialFeedbackRequest',
  full_name='cogment.TrialFeedbackRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.TrialFeedbackRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='feedbacks', full_name='cogment.TrialFeedbackRequest.feedbacks', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=829,
  serialized_end=907,
)


_TRIALFEEDBACKREPLY = _descriptor.Descriptor(
  name='TrialFeedbackReply',
  full_name='cogment.TrialFeedbackReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=909,
  serialized_end=929,
)


_TRIALMESSAGEREQUEST = _descriptor.Descriptor(
  name='TrialMessageRequest',
  full_name='cogment.TrialMessageRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.TrialMessageRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='messages', full_name='cogment.TrialMessageRequest.messages', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=931,
  serialized_end=1006,
)


_TRIALMESSAGEREPLY = _descriptor.Descriptor(
  name='TrialMessageReply',
  full_name='cogment.TrialMessageReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1008,
  serialized_end=1027,
)

_GETTRIALPARAMSREPLY.fields_by_name['params'].message_type = cogment_dot_api_dot_common__pb2._TRIALPARAMS
_TRIALSTARTREQUEST.fields_by_name['config'].message_type = cogment_dot_api_dot_common__pb2._TRIALCONFIG
_TRIALSTARTREPLY.fields_by_name['human_config'].message_type = cogment_dot_api_dot_common__pb2._ACTORCONFIG
_TRIALSTARTREPLY.fields_by_name['observation'].message_type = cogment_dot_api_dot_common__pb2._OBSERVATION
_TRIALACTIONREQUEST.fields_by_name['action'].message_type = cogment_dot_api_dot_common__pb2._ACTION
_TRIALACTIONREPLY.fields_by_name['observation'].message_type = cogment_dot_api_dot_common__pb2._OBSERVATION
_TRIALACTIONREPLY.fields_by_name['reward'].message_type = cogment_dot_api_dot_common__pb2._REWARD
_TRIALACTIONREPLY.fields_by_name['messages'].message_type = cogment_dot_api_dot_common__pb2._MESSAGECOLLECTION
_TRIALFEEDBACKREQUEST.fields_by_name['feedbacks'].message_type = cogment_dot_api_dot_common__pb2._FEEDBACK
_TRIALMESSAGEREQUEST.fields_by_name['messages'].message_type = cogment_dot_api_dot_common__pb2._MESSAGE
DESCRIPTOR.message_types_by_name['GetTrialParamsRequest'] = _GETTRIALPARAMSREQUEST
DESCRIPTOR.message_types_by_name['GetTrialParamsReply'] = _GETTRIALPARAMSREPLY
DESCRIPTOR.message_types_by_name['TrialHeartbeatRequest'] = _TRIALHEARTBEATREQUEST
DESCRIPTOR.message_types_by_name['TrialHeartbeatReply'] = _TRIALHEARTBEATREPLY
DESCRIPTOR.message_types_by_name['TrialStartRequest'] = _TRIALSTARTREQUEST
DESCRIPTOR.message_types_by_name['TrialJoinRequest'] = _TRIALJOINREQUEST
DESCRIPTOR.message_types_by_name['TrialStartReply'] = _TRIALSTARTREPLY
DESCRIPTOR.message_types_by_name['TrialEndRequest'] = _TRIALENDREQUEST
DESCRIPTOR.message_types_by_name['TrialEndReply'] = _TRIALENDREPLY
DESCRIPTOR.message_types_by_name['TrialActionRequest'] = _TRIALACTIONREQUEST
DESCRIPTOR.message_types_by_name['TrialActionReply'] = _TRIALACTIONREPLY
DESCRIPTOR.message_types_by_name['TrialFeedbackRequest'] = _TRIALFEEDBACKREQUEST
DESCRIPTOR.message_types_by_name['TrialFeedbackReply'] = _TRIALFEEDBACKREPLY
DESCRIPTOR.message_types_by_name['TrialMessageRequest'] = _TRIALMESSAGEREQUEST
DESCRIPTOR.message_types_by_name['TrialMessageReply'] = _TRIALMESSAGEREPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetTrialParamsRequest = _reflection.GeneratedProtocolMessageType('GetTrialParamsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTRIALPARAMSREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.GetTrialParamsRequest)
  })
_sym_db.RegisterMessage(GetTrialParamsRequest)

GetTrialParamsReply = _reflection.GeneratedProtocolMessageType('GetTrialParamsReply', (_message.Message,), {
  'DESCRIPTOR' : _GETTRIALPARAMSREPLY,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.GetTrialParamsReply)
  })
_sym_db.RegisterMessage(GetTrialParamsReply)

TrialHeartbeatRequest = _reflection.GeneratedProtocolMessageType('TrialHeartbeatRequest', (_message.Message,), {
  'DESCRIPTOR' : _TRIALHEARTBEATREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialHeartbeatRequest)
  })
_sym_db.RegisterMessage(TrialHeartbeatRequest)

TrialHeartbeatReply = _reflection.GeneratedProtocolMessageType('TrialHeartbeatReply', (_message.Message,), {
  'DESCRIPTOR' : _TRIALHEARTBEATREPLY,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialHeartbeatReply)
  })
_sym_db.RegisterMessage(TrialHeartbeatReply)

TrialStartRequest = _reflection.GeneratedProtocolMessageType('TrialStartRequest', (_message.Message,), {
  'DESCRIPTOR' : _TRIALSTARTREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialStartRequest)
  })
_sym_db.RegisterMessage(TrialStartRequest)

TrialJoinRequest = _reflection.GeneratedProtocolMessageType('TrialJoinRequest', (_message.Message,), {
  'DESCRIPTOR' : _TRIALJOINREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialJoinRequest)
  })
_sym_db.RegisterMessage(TrialJoinRequest)

TrialStartReply = _reflection.GeneratedProtocolMessageType('TrialStartReply', (_message.Message,), {
  'DESCRIPTOR' : _TRIALSTARTREPLY,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialStartReply)
  })
_sym_db.RegisterMessage(TrialStartReply)

TrialEndRequest = _reflection.GeneratedProtocolMessageType('TrialEndRequest', (_message.Message,), {
  'DESCRIPTOR' : _TRIALENDREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialEndRequest)
  })
_sym_db.RegisterMessage(TrialEndRequest)

TrialEndReply = _reflection.GeneratedProtocolMessageType('TrialEndReply', (_message.Message,), {
  'DESCRIPTOR' : _TRIALENDREPLY,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialEndReply)
  })
_sym_db.RegisterMessage(TrialEndReply)

TrialActionRequest = _reflection.GeneratedProtocolMessageType('TrialActionRequest', (_message.Message,), {
  'DESCRIPTOR' : _TRIALACTIONREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialActionRequest)
  })
_sym_db.RegisterMessage(TrialActionRequest)

TrialActionReply = _reflection.GeneratedProtocolMessageType('TrialActionReply', (_message.Message,), {
  'DESCRIPTOR' : _TRIALACTIONREPLY,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialActionReply)
  })
_sym_db.RegisterMessage(TrialActionReply)

TrialFeedbackRequest = _reflection.GeneratedProtocolMessageType('TrialFeedbackRequest', (_message.Message,), {
  'DESCRIPTOR' : _TRIALFEEDBACKREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialFeedbackRequest)
  })
_sym_db.RegisterMessage(TrialFeedbackRequest)

TrialFeedbackReply = _reflection.GeneratedProtocolMessageType('TrialFeedbackReply', (_message.Message,), {
  'DESCRIPTOR' : _TRIALFEEDBACKREPLY,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialFeedbackReply)
  })
_sym_db.RegisterMessage(TrialFeedbackReply)

TrialMessageRequest = _reflection.GeneratedProtocolMessageType('TrialMessageRequest', (_message.Message,), {
  'DESCRIPTOR' : _TRIALMESSAGEREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialMessageRequest)
  })
_sym_db.RegisterMessage(TrialMessageRequest)

TrialMessageReply = _reflection.GeneratedProtocolMessageType('TrialMessageReply', (_message.Message,), {
  'DESCRIPTOR' : _TRIALMESSAGEREPLY,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialMessageReply)
  })
_sym_db.RegisterMessage(TrialMessageReply)



_TRIAL = _descriptor.ServiceDescriptor(
  name='Trial',
  full_name='cogment.Trial',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=1030,
  serialized_end=1742,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetTrialParams',
    full_name='cogment.Trial.GetTrialParams',
    index=0,
    containing_service=None,
    input_type=_GETTRIALPARAMSREQUEST,
    output_type=_GETTRIALPARAMSREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Start',
    full_name='cogment.Trial.Start',
    index=1,
    containing_service=None,
    input_type=_TRIALSTARTREQUEST,
    output_type=_TRIALSTARTREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Join',
    full_name='cogment.Trial.Join',
    index=2,
    containing_service=None,
    input_type=_TRIALJOINREQUEST,
    output_type=_TRIALSTARTREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='End',
    full_name='cogment.Trial.End',
    index=3,
    containing_service=None,
    input_type=_TRIALENDREQUEST,
    output_type=_TRIALENDREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Action',
    full_name='cogment.Trial.Action',
    index=4,
    containing_service=None,
    input_type=_TRIALACTIONREQUEST,
    output_type=_TRIALACTIONREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ActionStream',
    full_name='cogment.Trial.ActionStream',
    index=5,
    containing_service=None,
    input_type=_TRIALACTIONREQUEST,
    output_type=_TRIALACTIONREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Heartbeat',
    full_name='cogment.Trial.Heartbeat',
    index=6,
    containing_service=None,
    input_type=_TRIALHEARTBEATREQUEST,
    output_type=_TRIALHEARTBEATREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GiveFeedback',
    full_name='cogment.Trial.GiveFeedback',
    index=7,
    containing_service=None,
    input_type=_TRIALFEEDBACKREQUEST,
    output_type=_TRIALFEEDBACKREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Version',
    full_name='cogment.Trial.Version',
    index=8,
    containing_service=None,
    input_type=cogment_dot_api_dot_common__pb2._VERSIONREQUEST,
    output_type=cogment_dot_api_dot_common__pb2._VERSIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SendMessage',
    full_name='cogment.Trial.SendMessage',
    index=9,
    containing_service=None,
    input_type=_TRIALMESSAGEREQUEST,
    output_type=_TRIALMESSAGEREPLY,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_TRIAL)

DESCRIPTOR.services_by_name['Trial'] = _TRIAL

# @@protoc_insertion_point(module_scope)



from abc import ABC, abstractmethod
import traceback
import atexit
from time import time

from cogment.api.environment_pb2_grpc import EnvironmentServicer as Servicer
from cogment.api.environment_pb2 import (EnvStartRequest, EnvStartReply,
                                         EnvUpdateReply, EnvEndReply, EnvOnMessageReply)
from cogment.api.common_pb2 import Feedback, ObservationData, Message
from cogment.utils import list_versions

from types import SimpleNamespace, ModuleType
from typing import Any, Dict, Tuple

from cogment.trial import Trial

import importlib


class ObservationProxy:
    _payload = None
    _is_snap = False

    def _get_snapshot(self, snap_type):
        if self._payload and self._is_snap:
            return self._payload

        self._is_snap = True
        self._payload = snap_type()
        return self._payload

    def _set_snapshot(self, v):
        self._is_snap = True
        self._payload = v

    def _get_delta(self, dt_type):
        if self._payload and not self._is_snap:
            return self._payload

        self._is_snap = False
        self._payload = dt_type()
        return self._payload

    def _set_delta(self, v):
        self._is_snap = False
        self._payload = v


def new_actions_table(settings, trial):
    actions_by_actor_class = settings.ActionsTable(trial)
    actions_by_actor_id = actions_by_actor_class.all_actions()

    return actions_by_actor_class, actions_by_actor_id


def pack_observations(obs, obs_set):
    """
    This converts a project-specific ObservationTable,containing a single
    observation per actor, into the cogment api ObservationSet, which contains
    a list of observations and a mapping specifying which observation each actor
    uses.

    Observations are automatically dedupped, if two actors share the same
    observation, only one will be on the wire.
    """
    seen_observations = {}

    for agent_observation in obs.all_observations():
        if not agent_observation._payload:
            raise Exception("An agent is missing an observation")
        obs_id = id(agent_observation._payload)

        # The index of the observation for this actor in the observation set.
        obs_key = seen_observations.get(obs_id)
        if obs_key is None:
            obs_key = len(obs_set.observations)

            obs_set.observations.append(ObservationData(
                content=agent_observation._payload.SerializeToString(),
                snapshot=agent_observation._is_snap
            ))

            seen_observations[obs_id] = obs_key

        obs_set.actors_map.append(obs_key)


class Environment(ABC):
    VERSIONS: Dict[str, str]

    def __init__(self, trial: Trial):
        self.trial = trial
        self.end_trial = False

    @abstractmethod
    def start(self, config):
        pass

    def end(self):
        pass

    @abstractmethod
    def update(self, actions):
        pass

    @abstractmethod
    def on_message(self, onmessage):
        pass


class EnvService(Servicer):

    def __init__(self, env_class, settings):
        assert issubclass(env_class, Environment)

        # We will be managing a pool of environments, keyed by their trial id.
        self._envs: Dict[str, Tuple[Any, Trial]] = {}
        self._env_config_type = settings.environment.config_type
        self._env_class = env_class
        self.settings: ModuleType = settings

        atexit.register(self._cleanup)

        print("Environment service started")

    # The orchestrator is requesting a new environment
    def Start(self, request, context):
        try:
            trial_id = request.trial_id
            if not trial_id:
                raise Exception("You must send a trial_id")
            if trial_id in self._envs:
                raise Exception("trial already exists")

            print(f"spinning up new environment: {trial_id}")

            trial_config = None
            if request.HasField("trial_config"):
                if self.settings.trial.config_type is None:
                    raise Exception("trial config data but no config type")
                trial_config = self.settings.trial.config_type()
                trial_config.ParseFromString(request.trial_config.content)

            # Instantiate the fresh environment
            trial = Trial(trial_id, self.settings,
                          request.actor_counts, trial_config)

            # build an action table.
            actions_by_actor_class, actions_by_actor_id = new_actions_table(
                self.settings, trial)

            trial.actions = actions_by_actor_class
            trial.actions_by_actor_id = actions_by_actor_id

            config = None
            if request.HasField("config"):
                if self._env_config_type is None:
                    raise Exception(
                        "This environment isn't expecting a config")

                config = self._env_config_type()
                config.ParseFromString(request.config.content)

            instance = self._env_class(trial)
            initial_observation = instance.start(config)

            self._envs[trial.id] = (instance, trial)

            # Send the initial state of the environment back to the client
            # (orchestrator, normally.)
            reply = EnvStartReply()
            reply.observation_set.tick_id = 0
            reply.observation_set.timestamp = int(time() * 1000000000)

            pack_observations(initial_observation, reply.observation_set)

            return reply

        except Exception:
            traceback.print_exc()
            raise

    def End(self, request, context):
        try:
            try:
                instance, trial = self._envs[request.trial_id]
                instance.end()
                del self._envs[request.trial_id]
                return EnvEndReply()
            except KeyError as err:
                raise Exception(f"trial [{request.trial_id}] does not exist. "
                                " This might be normal if you just reloaded the code of your Environments.")
        except Exception:
            traceback.print_exc()
            raise

    # The orchestrator is ready for the environment to move forward in time.
    def Update(self, request, context):
        try:
            try:
                instance, trial = self._envs[request.trial_id]
            except KeyError as err:
                raise Exception(f"trial [{request.trial_id}] does not exist")

            len_actions = len(request.action_set.actions)
            len_actors = len(trial.actions_by_actor_id)
            if len_actions != len_actors:
                raise Exception(f"Received {len_actions} actions but have {len_actors} actors")

            for i, action in enumerate(trial.actions_by_actor_id):
                action.ParseFromString(request.action_set.actions[i])

            # Advance time
            observations = instance.update(trial.actions)

            # This must be done AFTER the update, as calls to
            # actor.add_feedback must refer to the past.
            trial.tick_id += 1

            # Send the reply to the requestor.
            reply = EnvUpdateReply()
            reply.end_trial = instance.end_trial
            reply.observation_set.tick_id = trial.tick_id

            pack_observations(observations,
                              reply.observation_set)

            reply.feedbacks.extend(trial._get_all_feedback())

            reply.messages.extend(trial._get_all_messages(-1))

            reply.observation_set.timestamp = int(time() * 1000000000)
            return reply
        except Exception:
            traceback.print_exc()
            raise

    def OnMessage(self, request, context):
        try:
            try:
                instance, trial = self._envs[request.trial_id]

            except KeyError as err:
                raise Exception(f"trial [{request.trial_id}] does not exist")

            for m in request.messages:
                class_type = m.payload.type_url.split('.')
                user_data = getattr(importlib.import_module(
                    self.settings.protolib), class_type[-1])()
                m.payload.Unpack(user_data)
                instance.on_message(m.sender_id, user_data)

            reply = EnvOnMessageReply()

            return reply
        except Exception:
            traceback.print_exc()
            raise

    def Version(self, request, context):
        try:
            return list_versions(self._env_class)
        except Exception:
            traceback.print_exc()
            raise

    def _cleanup(self):
        for instance, _ in self._envs.values():
            instance.end()

        self._envs.clear()

        atexit.unregister(self._cleanup)



from abc import ABC, abstractmethod

from types import SimpleNamespace
from typing import Dict
import traceback
import atexit

from cogment.api.agent_pb2_grpc import AgentServicer

from cogment.api.agent_pb2 import (
    AgentStartReply, AgentDecideReply, AgentRewardReply, AgentEndReply, AgentOnMessageReply)

from cogment.utils import list_versions
from cogment.trial import Trial, Actor
from cogment.delta_encoding import DecodeObservationData

import importlib


class Agent(ABC):
    VERSIONS: Dict[str, str]

    def __init__(self, trial: Trial, actor: Actor, config):
        self.trial = trial
        self.config = config
        self.actor_id = actor.actor_id
        self.id_in_class = actor.id_in_class

    @abstractmethod
    def end(self, final_data):
        pass

    @abstractmethod
    def reward(self, reward):
        pass

    @abstractmethod
    def decide(self, observation):
        pass

    @abstractmethod
    def on_message(self, onmessage):
        pass


def trial_key(trial_id, actor_id):
    return f'{trial_id}_{actor_id}'


class AgentService(AgentServicer):

    def __init__(self, agent_class, settings):
        assert issubclass(agent_class, Agent)

        # We will be managing a pool of agents, keyed by their session id.
        self._agents: Dict[str, SimpleNamespace] = {}
        self._agent_class = agent_class
        self.settings: ModuleType = settings

        try:
            self._actor_class = agent_class.actor_class
        except AttributeError as err:
            raise AttributeError(
                "You must define an actor_class property to your Agent")

        atexit.register(self._cleanup)

        print("Agent Service started")

    def Start(self, request, context):
        try:
            trial_id = request.trial_id
            actor_id = request.actor_id

            if not trial_id:
                raise Exception("No trial_id provided")

            # Sanity check: We should only ever create a session once.
            if trial_id in self._agents:
                raise Exception("session already exists")

            trial_config = None
            if request.HasField("trial_config"):
                if self.settings.trial.config_type is None:
                    raise Exception("trial config data but no config type")
                trial_config = self.settings.trial.config_type()
                trial_config.ParseFromString(request.trial_config.content)

            trial = Trial(trial_id, self.settings,
                          request.actor_counts, trial_config)

            config = None
            if request.HasField("config"):
                if self._actor_class.config_type is None:
                    raise Exception("This actor isn't expecting a config")

                config = self._actor_class.config_type()
                config.ParseFromString(request.config.content)

            # Instantiate the fresh agent
            instance = self._agent_class(
                trial=trial,
                actor=trial.actors.all[actor_id],
                config=config)

            self._agents[trial_key(trial_id, actor_id)] = SimpleNamespace(
                instance=instance, trial=trial, last_observation=None)

            reply = AgentStartReply()

            return reply
        except Exception:
            traceback.print_exc()
            raise

    def End(self, request, context):
        try:
            try:
                key = trial_key(request.trial_id, request.actor_id)
                data = self._agents[key]
                data.instance.end(request.final_data)
                del self._agents[key]
                return AgentEndReply()
            except KeyError:
                raise Exception(f"trial [{request.trial_id}] or actor [{request.actor_id}] do not exist. "
                                "This might be normal if you just reloaded the code of your Agents.")
        except Exception:
            traceback.print_exc()
            raise

    # The orchestrator is ready for the environemnt to move forward in time.
    def Decide(self, request, context):

        try:
            try:
                data = self._agents[trial_key(request.trial_id,
                                              request.actor_id)]
            except KeyError:
                raise Exception(f"trial [{request.trial_id}] or actor [{request.actor_id}] do not exist")

            data.trial.tick_id = request.observation.tick_id
            data.last_observation = DecodeObservationData(
                self._agent_class.actor_class,
                request.observation.data,
                data.last_observation
            )

            action = data.instance.decide(data.last_observation)

            reply = AgentDecideReply()
            reply.action.content = action.SerializeToString()
            reply.feedbacks.extend(data.trial._get_all_feedback())

            reply.messages.extend(
                data.trial._get_all_messages(data.instance.actor_id))

            return reply
        except Exception:
            traceback.print_exc()
            raise

    def Reward(self, request, context):
        try:
            try:
                data = self._agents[trial_key(request.trial_id,
                                              request.actor_id)]
            except KeyError:
                raise Exception(f"trial [{request.trial_id}] or actor [{request.actor_id}] do not exist")

            data.instance.reward(request.reward)

            reply = AgentRewardReply()

            return reply
        except Exception:
            traceback.print_exc()
            raise

    def OnMessage(self, request, context):
        try:
            try:
                data = self._agents[trial_key(request.trial_id,
                                              request.actor_id)]
            except KeyError:
                raise Exception(f"trial [{request.trial_id}] or actor [{request.actor_id}] do not exist")

            for m in request.messages:
                class_type = m.payload.type_url.split('.')
                user_data = getattr(importlib.import_module(
                    self.settings.protolib), class_type[-1])()
                m.payload.Unpack(user_data)
                data.instance.on_message(m.sender_id, user_data)

            reply = AgentOnMessageReply()

            return reply
        except Exception:
            traceback.print_exc()
            raise

    def Version(self, request, context):
        try:
            return list_versions(self._agent_class)
        except Exception:
            traceback.print_exc()
            raise

    def _cleanup(self):
        for data in self._agents.values():
            data.instance.end()

        self._agents.clear()

        atexit.unregister(self._cleanup)



import os
import ast
import sys
import time
import logging
import threading
import traceback
import subprocess

from itertools import chain


def iteritems(d, *args, **kwargs):
    return iter(d.items(*args, **kwargs))


_logger = None


def _has_level_handler(logger):
    """Check if there is a handler in the logging chain that will handle
    the given logger's effective level.
    """
    level = logger.getEffectiveLevel()
    current = logger

    while current:
        if any(handler.level <= level for handler in current.handlers):
            return True

        if not current.propagate:
            break

        current = current.parent

    return False


def _log(type, message, *args, **kwargs):
    """Log a message to the 'werkzeug' logger.

    The logger is created the first time it is needed. If there is no
    level set, it is set to :data:`logging.INFO`. If there is no handler
    for the logger's effective level, a :class:`logging.StreamHandler`
    is added.
    """
    global _logger

    if _logger is None:
        _logger = logging.getLogger("reloader")

        if _logger.level == logging.NOTSET:
            _logger.setLevel(logging.INFO)

        if not _has_level_handler(_logger):
            _logger.addHandler(logging.StreamHandler())

    getattr(_logger, type)(message.rstrip(), *args, **kwargs)


def _iter_module_files():
    """This iterates over all relevant Python files.  It goes through all
    loaded files from modules, all files in folders of already loaded modules
    as well as all files reachable through a package.
    """
    # The list call is necessary on Python 3 in case the module
    # dictionary modifies during iteration.
    for module in list(sys.modules.values()):
        if module is None:
            continue
        filename = getattr(module, "__file__", None)
        if filename:
            if os.path.isdir(filename) and os.path.exists(
                    os.path.join(filename, "__init__.py")
            ):
                filename = os.path.join(filename, "__init__.py")

            old = None
            while not os.path.isfile(filename):
                old = filename
                filename = os.path.dirname(filename)
                if filename == old:
                    break
            else:
                if filename[-4:] in (".pyc", ".pyo"):
                    filename = filename[:-1]
                yield filename


def _find_observable_paths(extra_files=None):
    """Finds all paths that should be observed."""
    rv = set(
        os.path.dirname(os.path.abspath(x)) if os.path.isfile(x) else os.path.abspath(x)
        for x in sys.path
    )

    for filename in extra_files or ():
        rv.add(os.path.dirname(os.path.abspath(filename)))

    for module in list(sys.modules.values()):
        fn = getattr(module, "__file__", None)
        if fn is None:
            continue
        fn = os.path.abspath(fn)
        rv.add(os.path.dirname(fn))

    return _find_common_roots(rv)


def _get_args_for_reloading():
    """Determine how the script was executed, and return the args needed
    to execute it again in a new process.
    """
    rv = [sys.executable]
    py_script = sys.argv[0]
    args = sys.argv[1:]
    # Need to look at main module to determine how it was executed.
    __main__ = sys.modules["__main__"]

    # The value of __package__ indicates how Python was called. It may
    # not exist if a setuptools script is installed as an egg. It may be
    # set incorrectly for entry points created with pip on Windows.
    if getattr(__main__, "__package__", None) is None or (
            os.name == "nt"
            and __main__.__package__ == ""
            and not os.path.exists(py_script)
            and os.path.exists(py_script + ".exe")
    ):
        # Executed a file, like "python app.py".
        py_script = os.path.abspath(py_script)

        if os.name == "nt":
            # Windows entry points have ".exe" extension and should be
            # called directly.
            if not os.path.exists(py_script) and os.path.exists(py_script + ".exe"):
                py_script += ".exe"

            if (
                    os.path.splitext(sys.executable)[1] == ".exe"
                    and os.path.splitext(py_script)[1] == ".exe"
            ):
                rv.pop(0)

        rv.append(py_script)
    else:
        # Executed a module, like "python -m werkzeug.serving".
        if sys.argv[0] == "-m":
            # Flask works around previous behavior by putting
            # "-m flask" in sys.argv.
            # TODO remove this once Flask no longer misbehaves
            args = sys.argv
        else:
            if os.path.isfile(py_script):
                # Rewritten by Python from "-m script" to "/path/to/script.py".
                py_module = __main__.__package__
                name = os.path.splitext(os.path.basename(py_script))[0]

                if name != "__main__":
                    py_module += "." + name
            else:
                # Incorrectly rewritten by pydevd debugger from "-m script" to "script".
                py_module = py_script

            rv.extend(("-m", py_module.lstrip(".")))

    rv.extend(args)
    return rv


def _find_common_roots(paths):
    """Out of some paths it finds the common roots that need monitoring."""
    paths = [x.split(os.path.sep) for x in paths]
    root = {}
    for chunks in sorted(paths, key=len, reverse=True):
        node = root
        for chunk in chunks:
            node = node.setdefault(chunk, {})
        node.clear()

    rv = set()

    def _walk(node, path):
        for prefix, child in iteritems(node):
            _walk(child, path + (prefix,))
        if not node:
            rv.add("/".join(path))

    _walk(root, ())
    return rv


class ReloaderLoop(object):
    name = ""

    # monkeypatched by testsuite. wrapping with `staticmethod` is required in
    # case time.sleep has been replaced by a non-c function (e.g. by
    # `eventlet.monkey_patch`) before we get here
    _sleep = staticmethod(time.sleep)

    def __init__(self, extra_files=None, interval=1):
        self.extra_files = set(os.path.abspath(x) for x in extra_files or ())
        self.interval = interval

    def run(self):
        pass

    def restart_with_reloader(self):
        """Spawn a new Python interpreter with the same arguments as this one,
        but running the reloader thread.
        """
        while 1:
            _log("info", " * Restarting with %s" % self.name)
            args = _get_args_for_reloading()

            new_environ = os.environ.copy()

            new_environ["RUN_MAIN"] = "true"
            exit_code = subprocess.call(args, env=new_environ, close_fds=False)
            if exit_code != 3:
                return exit_code

    def trigger_reload(self, filename):
        _log("info", "Checking syntax before reloading. Filename: {}".format(filename))

        with open(filename) as source_file:
            try:
                ast.parse(source_file.read(), filename=filename)
            except SyntaxError:
                traceback.print_exc()
                return

        self.log_reload(filename)
        sys.exit(3)

    def log_reload(self, filename):
        filename = os.path.abspath(filename)
        _log("info", " * Detected change in %r, reloading" % filename)


class StatReloaderLoop(ReloaderLoop):
    name = "stat"

    def run(self):
        mtimes = {}
        while 1:
            for filename in chain(_iter_module_files(), self.extra_files):
                try:
                    mtime = os.stat(filename).st_mtime
                except OSError:
                    continue

                old_time = mtimes.get(filename)
                mtimes[filename] = mtime

                if old_time is None:
                    continue
                elif mtime > old_time:
                    self.trigger_reload(filename)

            self._sleep(self.interval)


def ensure_echo_on():
    """Ensure that echo mode is enabled. Some tools such as PDB disable
    it which causes usability issues after reload."""
    # tcgetattr will fail if stdin isn't a tty
    if not sys.stdin.isatty():
        return
    try:
        import termios
    except ImportError:
        return
    attributes = termios.tcgetattr(sys.stdin)
    if not attributes[3] & termios.ECHO:
        attributes[3] |= termios.ECHO
        termios.tcsetattr(sys.stdin, termios.TCSANOW, attributes)


def run_with_reloader(main_func, *args):
    """Run the given function in an independent python interpreter."""
    import signal

    reloader = StatReloaderLoop()

    signal.signal(signal.SIGTERM, lambda _: sys.exit(0))
    try:
        if os.environ.get("RUN_MAIN") == "true":
            ensure_echo_on()
            t = threading.Thread(target=main_func, args=())
            t.setDaemon(True)
            t.start()
            reloader.run()
        else:
            sys.exit(reloader.restart_with_reloader())
    except KeyboardInterrupt:
        pass



class EnvClass:
    def __init__(self, id, config_type, message_space):
        self.id = id
        self.config_type = config_type
        self.message_space = message_space


class EnvClassList:
    def __init__(self, *args):
        self._env_classes_list = list(args)

        for a_c in args:
            setattr(self, a_c.id, a_c)

    def __iter__(self):
        return iter(self._env_classes_list)

    def get_env_counts(self, trial_params):
        result = [0] * len(self._env_classes_list)

        index_map = {}
        for index, env_class in enumerate(self._env_classes_list):
            index_map[env_class.id] = index

        for env in trial_params.actors:
            result[index_map[env.env_class]] += 1

        return result



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from cogment.api import data_pb2 as cogment_dot_api_dot_data__pb2


class LogExporterStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Log = channel.stream_unary(
        '/cogment.LogExporter/Log',
        request_serializer=cogment_dot_api_dot_data__pb2.DatalogMsg.SerializeToString,
        response_deserializer=cogment_dot_api_dot_data__pb2.LogReply.FromString,
        )


class LogExporterServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def Log(self, request_iterator, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_LogExporterServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Log': grpc.stream_unary_rpc_method_handler(
          servicer.Log,
          request_deserializer=cogment_dot_api_dot_data__pb2.DatalogMsg.FromString,
          response_serializer=cogment_dot_api_dot_data__pb2.LogReply.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'cogment.LogExporter', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Copyright 2019 Age of Minds inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""cogment_sdk packaging."""

from setuptools import setup

with open("README.md", "r") as fh:
    LONG_DESCRIPTION = fh.read()

version = {}
with open("cogment/version.py") as fp:
    exec(fp.read(), version)

setup(name='cogment',
      version=version['__version__'],
      description='Cogment SDK',
      url='https://gitlab.com/cogment/cogment',
      long_description=LONG_DESCRIPTION,
      long_description_content_type="text/markdown",
      author='Artificial Intelligence Redefined',
      author_email='dev+cogment@ai-r.com',
      license='Apache License 2.0',
      packages=['cogment', 'cogment.api'],
      tests_require=['pytest'],
      install_requires=[
          'grpcio>=1.19',
          'grpcio-reflection>=1.19',
          'grpcio-tools>=1.19',
          'protobuf>=3.7'
      ],)
